@@ -1,4 +1,4 @@
-// Copyright 2021-2024 Sebastian Ramacher
+// Copyright 2021-2025 Sebastian Ramacher
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 use std::{
@@ -33,6 +33,7 @@ pub(crate) fn default_progress_template() -> &'static str {
     "{msg}: {spinner:.green} [{wide_bar:.cyan/blue}] {pos}/{len} ({per_sec}, {eta})"
 }
 
+#[allow(dead_code)]
 pub(crate) enum CacheEntries {
     Excuses,
     Packages(Suite),
@@ -327,9 +328,8 @@ impl Cache {
 
     fn lookup_url(&self, suite: Suite, path: &str) -> String {
         format!(
-            "{}/dists/{}/{}",
+            "{}/dists/{suite}/{}",
             self.archive_mirror,
-            suite,
             match suite {
                 Suite::Unstable => &self.unstable,
                 Suite::Testing(_) => &self.testing,
@@ -477,9 +477,11 @@ impl Cache {
         Ok(all_paths)
     }
 
+    /*
     pub fn get_source_path(&self, suite: Suite) -> Result<PathBuf> {
         self.get_cache_path(format!("Sources_{suite}"))
     }
+    */
 
     // Architectures for a suite (including Arch: all)
     pub fn architectures_for_suite(&self, suite: Suite) -> Vec<Architecture> {
