@@ -1,4 +1,4 @@
-// Copyright 2021-2023 Sebastian Ramacher
+// Copyright 2021-2025 Sebastian Ramacher
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 use std::{
@@ -16,7 +16,7 @@ use assorted_debian_utils::{
     archive::{Codename, Extension, Suite, SuiteOrCodename},
     rfc822_like,
     version::PackageVersion,
-    wb::{BinNMU, SourceSpecifier, WBCommandBuilder},
+    wb::{BinNMU, SourceSpecifier, WBArchitecture, WBCommandBuilder},
 };
 use async_trait::async_trait;
 use clap::Parser;
@@ -29,6 +29,7 @@ use crate::{
     config::{
         default_progress_style, default_progress_template, source_skip_binnmu, Cache, CacheEntries,
     },
+    source_packages::{self, SourcePackages},
     udd_bugs::{load_bugs_from_reader, UDDBugs},
     utils::execute_wb_commands,
     AsyncCommand, BaseOptions, Downloads,
@@ -37,9 +38,8 @@ use crate::{
 #[derive(Deserialize, Debug, Eq, PartialEq)]
 #[serde(rename_all = "PascalCase")]
 struct BinaryPackage {
-    source: Option<String>,
-    package: String,
-    version: PackageVersion,
+    #[serde(flatten)]
+    package: source_packages::BinaryPackage,
     architecture: Architecture,
     #[serde(rename = "Built-Using")]
     built_using: Option<String>,
@@ -49,13 +49,6 @@ struct BinaryPackage {
     x_cargo_built_using: Option<String>,
 }
 
-#[derive(Deserialize, Debug, Eq, PartialEq)]
-#[serde(rename_all = "PascalCase")]
-struct SourcePackage {
-    package: String,
-    version: PackageVersion,
-}
-
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
 #[allow(clippy::enum_variant_names)]
 pub enum Field {
@@ -67,9 +60,9 @@ pub enum Field {
 impl fmt::Display for Field {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            Field::BuiltUsing => write!(f, "Built-Using"),
-            Field::StaticBuiltUsing => write!(f, "Static-Built-Using"),
-            Field::XCargoBuiltUsing => write!(f, "X-Cargo-Built-Using"),
+            Self::BuiltUsing => write!(f, "Built-Using"),
+            Self::StaticBuiltUsing => write!(f, "Static-Built-Using"),
+            Self::XCargoBuiltUsing => write!(f, "X-Cargo-Built-Using"),
         }
     }
 }
@@ -88,9 +81,9 @@ impl FromStr for Field {
 
     fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
         match s {
-            "Built-Using" => Ok(Field::BuiltUsing),
-            "Static-Built-Using" => Ok(Field::StaticBuiltUsing),
-            "X-Cargo-Built-Using" => Ok(Field::XCargoBuiltUsing),
+            "Built-Using" => Ok(Self::BuiltUsing),
+            "Static-Built-Using" => Ok(Self::StaticBuiltUsing),
+            "X-Cargo-Built-Using" => Ok(Self::XCargoBuiltUsing),
             _ => Err(ParseError),
         }
     }
@@ -104,9 +97,7 @@ pub(crate) struct NMUOutdatedBuiltUsingOptions {
     /// Suite for binNMUs.
     #[clap(short, long, default_value_t = SuiteOrCodename::Suite(Suite::Unstable))]
     suite: SuiteOrCodename,
-    /// Set architectures for binNMUs. If no archictures are specified, the binNMUs are scheduled with ANY.
-    #[clap(short, long)]
-    architecture: Option<Vec<Architecture>>,
+    /// Select the binary package field to check for outdated information. By default, the `Built-Using` field is checked.
     #[clap(long, default_value_t = Field::BuiltUsing)]
     field: Field,
 }
@@ -118,6 +109,7 @@ struct OutdatedPackage {
     suite: Suite,
     outdated_dependency: String,
     outdated_version: PackageVersion,
+    architecture: WBArchitecture,
 }
 
 #[derive(PartialEq, Eq)]
@@ -125,6 +117,7 @@ struct CombinedOutdatedPackage {
     source: String,
     version: PackageVersion,
     suite: Suite,
+    architecture: WBArchitecture,
     outdated_dependencies: Vec<(String, PackageVersion)>,
 }
 
@@ -154,11 +147,11 @@ fn split_dependency(dependency: &str) -> Option<(String, PackageVersion)> {
 struct BinaryPackageParser<'a> {
     field: Field,
     iterator: ProgressBarIter<IntoIter<BinaryPackage>>,
-    sources: &'a HashMap<String, PackageVersion>,
+    sources: &'a SourcePackages,
 }
 
 impl<'a> BinaryPackageParser<'a> {
-    fn new<P>(field: Field, sources: &'a HashMap<String, PackageVersion>, path: P) -> Result<Self>
+    fn new<P>(field: Field, sources: &'a SourcePackages, path: P) -> Result<Self>
     where
         P: AsRef<Path>,
     {
@@ -167,7 +160,7 @@ impl<'a> BinaryPackageParser<'a> {
         let pb = ProgressBar::new(binary_packages.len() as u64);
         pb.set_style(default_progress_style().template(default_progress_template())?);
         pb.set_message(format!("Processing {}", path.as_ref().display()));
-        // collect all sources with arch dependent binaries having Built-Using set and their Built-Using fields
+        // collect all sources with arch dependent binaries having Built-Using set and their Built-Using fields refer to ESO sources
         Ok(Self {
             field,
             iterator: binary_packages.into_iter().progress_with(pb),
@@ -176,8 +169,15 @@ impl<'a> BinaryPackageParser<'a> {
     }
 }
 
+struct OutdatedSourcePackage {
+    source: String,
+    version: PackageVersion,
+    built_using: HashSet<(String, PackageVersion)>,
+    architecture: WBArchitecture,
+}
+
 impl Iterator for BinaryPackageParser<'_> {
-    type Item = (String, PackageVersion, HashSet<(String, PackageVersion)>);
+    type Item = OutdatedSourcePackage;
 
     fn next(&mut self) -> Option<Self::Item> {
         while let Some(binary_package) = self.iterator.next() {
@@ -194,15 +194,7 @@ impl Iterator for BinaryPackageParser<'_> {
                 continue;
             };
 
-            let source_package = if let Some(source_package) = &binary_package.source {
-                match source_package.split_whitespace().next() {
-                    Some(package) => package,
-                    None => continue,
-                }
-            } else {
-                // no Source set, so Source == Package
-                &binary_package.package
-            };
+            let (source_package, version) = binary_package.package.name_and_version();
 
             // remove trailing spaces found in X-Cargo-Built-Using
             let built_using = built_using.strip_suffix(' ').unwrap_or(built_using);
@@ -216,13 +208,13 @@ impl Iterator for BinaryPackageParser<'_> {
                     if split.is_none() {
                         warn!(
                             "Package '{}' contains invalid dependency in {}: {}",
-                            binary_package.package, self.field, dependency
+                            binary_package.package.package, self.field, dependency
                         );
                     }
                     split
                 })
                 .filter(|(source, version)| {
-                    if let Some(max_version) = self.sources.get(source) {
+                    if let Some(max_version) = self.sources.version(source) {
                         version < max_version
                     } else {
                         // This can happen when considering packages with
@@ -230,7 +222,7 @@ impl Iterator for BinaryPackageParser<'_> {
                         // rebuild those packages in any case.
                         trace!(
                             "Package '{}' refers to non-existing source package '{}'.",
-                            binary_package.package,
+                            binary_package.package.package,
                             source
                         );
                         true
@@ -247,11 +239,18 @@ impl Iterator for BinaryPackageParser<'_> {
                 continue;
             }
 
-            return Some((
-                source_package.into(),
-                binary_package.version.without_binnmu_version(),
+            // if the package builds any MA: same packages, schedule binNMUs with ANY
+            let architecture = if self.sources.is_ma_same(source_package) {
+                WBArchitecture::Any
+            } else {
+                WBArchitecture::Architecture(binary_package.architecture)
+            };
+            return Some(OutdatedSourcePackage {
+                source: source_package.into(),
+                version: version.without_binnmu_version(),
                 built_using,
-            ));
+                architecture,
+            });
         }
 
         None
@@ -286,34 +285,14 @@ impl<'a> NMUOutdatedBuiltUsing<'a> {
         )
     }
 
-    /// Load source packages with the highest version
-    fn load_sources<P>(path: P, destination: &mut HashMap<String, PackageVersion>) -> Result<()>
-    where
-        P: AsRef<Path>,
-    {
-        let sources: Vec<SourcePackage> = rfc822_like::from_file(path.as_ref())?;
-
-        for source in sources {
-            destination
-                .entry(source.package)
-                .and_modify(|version| {
-                    if source.version > *version {
-                        *version = source.version.clone();
-                    }
-                })
-                .or_insert(source.version);
-        }
-
-        Ok(())
-    }
-
     /// Load source packages from multiple suites with the highest version
-    fn load_sources_for_suites(&self, suites: &[Suite]) -> Result<HashMap<String, PackageVersion>> {
-        let mut ret: HashMap<String, PackageVersion> = HashMap::default();
-        for suite in suites {
-            Self::load_sources(self.cache.get_source_path(*suite)?, &mut ret)?;
-        }
-        Ok(ret)
+    fn load_sources_for_suites(&self, suites: &[Suite]) -> Result<SourcePackages> {
+        let paths = suites
+            .iter()
+            .flat_map(|suite| self.cache.get_package_paths(*suite, false).into_iter())
+            .flatten()
+            .collect::<Vec<_>>();
+        SourcePackages::new(&paths)
     }
 
     fn load_eso(&self, field: Field, suite: Suite) -> Result<Vec<CombinedOutdatedPackage>> {
@@ -323,8 +302,12 @@ impl<'a> NMUOutdatedBuiltUsing<'a> {
         let mut packages = HashSet::new();
         for suite in self.expand_suite_for_binaries() {
             for path in self.cache.get_package_paths(suite, false)? {
-                for (source, version, dependencies) in
-                    BinaryPackageParser::new(field, &source_packages, path)?
+                for OutdatedSourcePackage {
+                    source,
+                    version,
+                    built_using: dependencies,
+                    architecture,
+                } in BinaryPackageParser::new(field, &source_packages, path)?
                 {
                     packages.extend(dependencies.into_iter().map(
                         |(outdated_dependency, outdated_version)| OutdatedPackage {
@@ -333,14 +316,17 @@ impl<'a> NMUOutdatedBuiltUsing<'a> {
                             suite,
                             outdated_dependency,
                             outdated_version,
+                            architecture,
                         },
                     ));
                 }
             }
         }
 
-        let mut result =
-            HashMap::<(String, Suite), HashSet<(PackageVersion, String, PackageVersion)>>::new();
+        let mut result = HashMap::<
+            (String, Suite, WBArchitecture),
+            HashSet<(PackageVersion, String, PackageVersion)>,
+        >::new();
         for outdated_package in packages {
             // skip some packages that make no sense to binNMU
             if source_skip_binnmu(&outdated_package.source) {
@@ -367,7 +353,11 @@ impl<'a> NMUOutdatedBuiltUsing<'a> {
             }
 
             result
-                .entry((outdated_package.source, outdated_package.suite))
+                .entry((
+                    outdated_package.source,
+                    outdated_package.suite,
+                    outdated_package.architecture,
+                ))
                 .or_default()
                 .insert((
                     outdated_package.version,
@@ -378,7 +368,7 @@ impl<'a> NMUOutdatedBuiltUsing<'a> {
 
         Ok(result
             .into_iter()
-            .filter_map(|((source, suite), outdated_dependencies)| {
+            .filter_map(|((source, suite, architecture), outdated_dependencies)| {
                 let max_version = outdated_dependencies
                     .iter()
                     .map(|(version, _, _)| version)
@@ -388,6 +378,7 @@ impl<'a> NMUOutdatedBuiltUsing<'a> {
                     source,
                     version: max_version.clone(),
                     suite,
+                    architecture,
                     outdated_dependencies: outdated_dependencies
                         .into_iter()
                         .filter_map(|(version, outdated_dependency, outdated_version)| {
@@ -448,9 +439,7 @@ impl AsyncCommand for NMUOutdatedBuiltUsing<'_> {
             let mut source = SourceSpecifier::new(&outdated_package.source);
             source.with_version(&outdated_package.version);
             source.with_suite(outdated_package.suite.into());
-            if let Some(architectures) = &self.options.architecture {
-                source.with_archive_architectures(architectures);
-            }
+            source.with_architectures(&[outdated_package.architecture]);
 
             let message = format!(
                 "Rebuild for outdated {} ({})",
@@ -480,11 +469,6 @@ impl Downloads for NMUOutdatedBuiltUsing<'_> {
         self.expand_suite_for_binaries()
             .into_iter()
             .map(CacheEntries::Packages)
-            .chain(
-                self.expand_suite_for_sources()
-                    .into_iter()
-                    .map(CacheEntries::Sources),
-            )
             .collect()
     }
 }
